package me.desht.pneumaticcraft.api.fuel;

import net.minecraft.fluid.Fluid;
import net.minecraft.tags.ITag;

import java.util.Collection;

public interface IFuelRegistry {
    /**
     * Adds a burnable fluid to the Liquid Compressor's available burnable fuels.  This also allows a bucket
     * of that fluid to be used in furnaces, the burn time being half the mLPerBucket value.
     *
     * @param fluid the fluid to register
     * @param mLPerBucket the amount of mL of air generated by burning 1000mB of the fuel. For comparison, one piece
     *                    of coal generates 16000mL in an Air Compressor.
     */
    void registerFuel(ITag<Fluid> fluid, int mLPerBucket);

    /**
     * Adds a burnable fluid to the Liquid Compressor's available burnable fuels.  This also allows a bucket
     * of that fluid to be used in furnaces, the burn time being half the mLPerBucket value.
     *
     * @param fluid the fluid to register
     * @param mLPerBucket the amount of mL of air generated by burning 1000mB of the fuel in a Liquid Compressor. For
     *                   comparison, one piece of coal generates 16000mL in an Air Compressor.
     * @param burnRateMultiplier fuel burn rate multiplier; larger numbers mean the fuel burns faster and produces air
     *                          more quickly.  The total amount of air produced does not change.
     */
    void registerFuel(ITag<Fluid> fluid, int mLPerBucket, float burnRateMultiplier);

    /**
     * Get the fuel value of the given fluid; this is defined as the amount of compressed air generated in a Liquid
     * Compressor (without Speed Upgrades) by burning 1000mL of the fluid.
     * @param fluid the fluid
     * @return the fuel value (0 for fluids which have not been registered as fuels)
     */
    int getFuelValue(Fluid fluid);

    /**
     * Get the burn rate of the given fluid. Higher burn rates will generate compressed air faster in a Liquid
     * Compressor (and be used up faster), without affecting the total amount of air produced.
     * @param fluid the fluid
     * @return the burn rate (1.0f is the standard, default, rate)
     */
    float getBurnRateMultiplier(Fluid fluid);

    /**
     * Get all the known fuels which have been registered with the fuel registry.
     *
     * @return a collection of fuel fluids
     */
    Collection<Fluid> registeredFuels();
}
